const _ = require('lodash')
const assert = require('assert')
const Faker = require('faker')
const process = require('process')
const path = require('path')
const target = process.env.NODE_ENV === 'production' ? 'dist' : 'libs'
const Rabbitmq = require(path.resolve(target, 'index.js'))
const err = new Error('test')
const routingKey = 'testroutingKey'
const exchange = 'testexchange'
const exchange2 = 'testexchange2'
const queue = 'testqueue'
let consumerTag
let cb
let result
let options
let content = 'test'
let message
let url

process.on('unhandledRejection', e => {})

/**
 *
 */
describe('TEST Rabbitmq', () => {
  it('RED: CASE1 when connection url is invalid, throws an error', done => {
    url = Rabbitmq.options.url
    Rabbitmq.options.url = null
    assert.rejects(Rabbitmq.connect.bind(Rabbitmq), 'invalid connection url!')

    done()
  })
  it('GREEN: CASE1 when connection url is invalid, throws an error', done => {
    Rabbitmq.options.url = url
    assert.doesNotReject(Rabbitmq.connect.bind(Rabbitmq))

    done()
  })
  it('GREEN: CASE2 helper functions', done => {
    result = Rabbitmq.isFatalError(err)
    assert.strictEqual(result, true)

    result = Rabbitmq.isIllegalOperationError(err)
    assert.strictEqual(result, false)

    err.name = 'IllegalOperationError'
    result = Rabbitmq.isIllegalOperationError(err)
    assert.strictEqual(result, true)

    Rabbitmq.onChannelDrain()
    Rabbitmq.onChannelError(err)
    Rabbitmq.onChannelReturned(err.message)
    Rabbitmq.onBlocked(err.message)
    Rabbitmq.onUnblocked()
    Rabbitmq.onConnectionError(err)

    done()
  })
  it('GREEN: CASE3 assertQueue', done => {
    Rabbitmq.assertQueue(queue).then(res => {
      assert.strictEqual(res.queue, queue)
      done()
    })
  })
  it('GREEN: CASE4 assertExchange', done => {
    assert.doesNotReject(Rabbitmq.assertExchange.bind(Rabbitmq, exchange))
    Rabbitmq.assertExchange(exchange2).then(res => {
      assert.strictEqual(res.exchange, exchange2)
      done()
    })
  })
  it('GREEN: CASE5 bindQueue', done => {
    Rabbitmq.bindQueue(queue, exchange, routingKey).then(res => {
      assert.strictEqual(_.isEmpty(res), true)
      done()
    })
  })
  it('GREEN: CASE6 publish', done => {
    options = {
      persistent: true,
      priority: 1
    }
    assert.doesNotReject(Rabbitmq.publish.bind(Rabbitmq, exchange, routingKey, content, options))

    content = {
      username: Faker.internet.userName(),
      password: Faker.internet.password()
    }
    Rabbitmq.publish(exchange, routingKey, content, options).then(res => {
      assert.strictEqual(_.isEmpty(res), true)
      done()
    })
  })
  it('GREEN: CASE7 prefetch', done => {
    Rabbitmq.prefetch(1).then(res => {
      assert.strictEqual(_.isEmpty(res), true)
      done()
    })
  })
  it('GREEN: CASE8 sendToQueue', done => {
    options = {
      presistent: true,
      replyTo: undefined
    }
    Rabbitmq.sendToQueue(queue, content, options).then(res => {
      assert.strictEqual(_.isBoolean(res), true)
      done()
    })
  })
  it('GREEN: CASE9 consume', done => {
    cb = args => {}
    options = {
      noAck: false
    }
    Rabbitmq.consume(queue, cb, options).then(res => {
      consumerTag = res.consumerTag
      assert.strictEqual(_.isString(consumerTag), true)
      done()
    })
  })
  it('GREEN: CASE10 recover', done => {
    Rabbitmq.recover().then(res => {
      assert.strictEqual(_.isEmpty(res), true)
      done()
    })
  })
  it('GREEN: CASE11 createConfirmChannel', done => {
    Rabbitmq.createConfirmChannel().then(res => {
      assert.strictEqual(_.isFunction(res.sendToQueue), true)
      done()
    })
  })
  it('GREEN: CASE12 assertTemporaryQueue', done => {
    options = {
      exclusive: true
    }
    Rabbitmq.assertTemporaryQueue(options).then(res => {
      assert.strictEqual(_.isString(res.queue), true)
      done()
    })
  })
  it('GREEN: CASE13 ackAll', done => {
    Rabbitmq.ackAll().then(res => {
      assert.strictEqual(_.isEmpty(res), true)
      done()
    })
  })
  it('GREEN: CASE14 nackAll', done => {
    Rabbitmq.nackAll().then(res => {
      assert.strictEqual(_.isEmpty(res), true)
      done()
    })
  })
  it('GREEN: CASE15 get', done => {
    options = {
      noAck: true
    }
    Rabbitmq.get(queue).then(res => {
      assert.strictEqual(_.isEmpty(res), true)
      done()
    })
  })
  it('GREEN: CASE16 checkQueue', done => {
    Rabbitmq.checkQueue(queue).then(res => {
      assert.strictEqual(res.queue, queue)
      done()
    })
  })
  it('GREEN: CASE17 checkExchange', done => {
    Rabbitmq.checkExchange(exchange).then(res => {
      assert.strictEqual(_.isEmpty(res), true)
      done()
    })
  })
  it('GREEN: CASE18 sendToQueue => subscribe and cancel a message', done => {
    options = {
      presistent: true,
      replyTo: undefined
    }
    Rabbitmq.sendToQueue(queue, content, options).then(res => {
      assert.strictEqual(_.isBoolean(res), true)
      cb = args => {}
      options = {
        noAck: false
      }
      Rabbitmq.subscribe(queue, cb, options).then(res => {
        consumerTag = res.consumerTag
        assert.strictEqual(_.isString(res.consumerTag), true)
        Rabbitmq.cancel(consumerTag).then(() => done())
      })
    })
  })
  it('GREEN: CASE19 sendToQueue => get and acknowledge a message', done => {
    options = {
      presistent: true,
      replyTo: undefined
    }
    Rabbitmq.sendToQueue(queue, content, options).then(res => {
      assert.strictEqual(_.isBoolean(res), true)
      Rabbitmq.get(queue).then(res => {
        assert.strictEqual(_.isEmpty(res), false)
        message = Rabbitmq.parseContent(res)
        message = Rabbitmq.parseContent(message)
        Rabbitmq.ack(res).then(res => {
          assert.strictEqual(_.isEmpty(res), true)
          done()
        })
      })
    })
  })
  it('GREEN: CASE20 sendToQueue => get and not acknowledge a message', done => {
    options = {
      presistent: true,
      replyTo: undefined
    }
    Rabbitmq.sendToQueue(queue, content, options).then(res => {
      assert.strictEqual(_.isBoolean(res), true)
      Rabbitmq.get(queue).then(res => {
        assert.strictEqual(_.isEmpty(res), false)
        Rabbitmq.nack(res, false, false).then(res => {
          assert.strictEqual(_.isEmpty(res), true)
          done()
        })
      })
    })
  })
  it('GREEN: CASE21 sendToQueue => get and not reject a message', done => {
    options = {
      presistent: true,
      replyTo: undefined
    }
    Rabbitmq.sendToQueue(queue, content, options).then(res => {
      assert.strictEqual(_.isBoolean(res), true)
      Rabbitmq.get(queue).then(res => {
        assert.strictEqual(_.isEmpty(res), false)
        Rabbitmq.reject(res, false).then(res => {
          assert.strictEqual(_.isEmpty(res), true)
          done()
        })
      })
    })
  })
  it('GREEN: CASE22 unbindQueue', done => {
    Rabbitmq.unbindQueue(queue, exchange, routingKey).then(res => {
      assert.strictEqual(_.isEmpty(res), true)
      done()
    })
  })
  it('GREEN: CASE21 bindExchange', done => {
    Rabbitmq.bindExchange(exchange2, exchange, routingKey).then(res => {
      assert.strictEqual(_.isEmpty(res), true)
      done()
    })
  })
  it('GREEN: CASE22 unbindExchange', done => {
    Rabbitmq.unbindExchange(exchange2, exchange, routingKey).then(res => {
      assert.strictEqual(_.isEmpty(res), true)
      done()
    })
  })
  after(async () => {
    await Rabbitmq.purgeQueue(queue)
    await Rabbitmq.deleteQueue(queue)
    await Rabbitmq.deleteExchange(exchange)
    await Rabbitmq.deleteExchange(exchange2)

    result = await Rabbitmq.disconnect()
    assert.strictEqual(result, true)

    result = await Rabbitmq.dispose()
    assert.strictEqual(result, true)
  })
})
